@@ -392,7 +392,7 @@ class Array(metaclass=MetaArray):
         return info
 
     @classmethod
-    def _from_buffer(cls, buffer, offset):
+    def _from_buffer(cls, buffer, offset=0):
         self = object.__new__(cls)
         self._buffer = buffer
         self._offset = offset
