@@ -306,11 +306,11 @@ class XBuffer(ABC):
 
     @abstractmethod
     def copy_to_native(self, dest, dest_offset, source_offset, nbytes):
-        """copy data from self.buffer into dest"""
+        """Copy data from self.buffer into dest"""
 
     @abstractmethod
-    def copy_native(self, offset, nbytes):
-        """return native data with content at from offset and nbytes"""
+    def to_native(self, offset, nbytes):
+        """Return native data with content at from offset and nbytes"""
 
     @abstractmethod
     def update_from_buffer(self, offset, source):
@@ -318,22 +318,22 @@ class XBuffer(ABC):
 
     @abstractmethod
     def to_nplike(self, offset, dtype, shape):
-        """view in nplike"""
+        """Return a view in of an nplike"""
 
     @abstractmethod
     def update_from_nplike(self, offset, dest_dtype, value):
-        """update data from nplike matching dest_dtype"""
+        """Copy data from nplike matching dest_dtype"""
 
     @abstractmethod
     def to_bytearray(self, offset, nbytes):
-        """copy in byte array: used in update_from_xbuffer"""
+        """Return a byte array: used in update_from_xbuffer"""
 
     @abstractmethod
     def to_pointer_arg(self, offset, nbytes):
-        """return data that can be used as argument in kernel"""
+        """Return data that can be used as argument in kernel"""
 
     def update_from_xbuffer(self, offset, source, source_offset, nbytes):
-        """update from any xbuffer, don't pass through gpu if possible"""
+        """Copy data from any xbuffer, don't pass through cpu if possible"""
         if source.context == self.context:
             self.update_from_native(
                 offset, source.buffer, source_offset, nbytes
